﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using PluginBase;

namespace SamplePlugin
{
    public class SamplePlugin : ISFCEditorPlugin
    {
        SampleROMDataSet ds;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public SamplePlugin()
        {
        }

        /// <summary>
        /// SFCGENEditorの代わりに表示するエディター名
        /// </summary>
        public string EditorName
        {
            get
            {
                return "SampleGameEditor";
            }
        }

        /// <summary>
        /// ターゲットのゲーム名 rommap.xmlと一致させる必要あり
        /// </summary>
        public string TargetROM
        {
            get
            {
                return "SampleGame";
            }
        }

        /// <summary>
        /// プラグインのバージョン番号
        /// </summary>
        public string Version
        {
            get
            {
                return "1.00";
            }
        }

        
        /// <summary>
        /// 未使用
        /// </summary>
        public string Description
        {
            get
            {
                return "";
            }
        }

        public string Token
        {
            get
            {
                return "";
            }
        }


        /// <summary>
        /// エディタープログラムとして表示するアイコン nullの場合はデフォルトを使用
        /// </summary>
        public Icon PluginIcon
        {
            get
            {
                return null;
            }
        }

        /// <summary>
        /// プラグインのホスト(呼び出し側)のセット 
        /// </summary>
        public ISFCEditorPluginHost PluginHost
        {
            get
            {
                return ROMDataSet.PluginHost;
            }
            set
            {
                ROMDataSet.PluginHost = value;
            }
        }


        /// <summary>
        /// プラグインロード時にSFCGENEditorから呼ばれる
        /// </summary>
        /// <param name="rds"></param>
        public void Init(ref ROMDataSet rds)
        {
            ds = new SampleROMDataSet(rds);
            rds = (ROMDataSet)ds;
/*
            //BRK(#$00)の解釈を変える場合に必要 SFCDQ3,6は BRKの後に2バイトの引数を
            //メッセージIDとして取っているために、直後2バイトを引数扱いにしないと正しく逆アセンブルできない
            //COP(#$02)の解釈を変える場合は同じようにしてMnemonic65816インスタンスを作成し、SFCDisasm.SetCOPをコールする
            Mnemonic65816 brk = new Mnemonic65816D();
            brk.Mnemonic="BRK";
            brk.ArgSize = 2;
            brk.at = ArgType.Const;
            brk.Contents = "goto $00FFE6(メッセージ表示) ";
            SFCDisasm.SetBRK(brk);

    */

        }

        /// <summary>
        /// ROMロード時に解釈のために呼ばれる
        /// </summary>
        /// <param name="NodeCount"></param>
        public void ParseROM(ref int NodeCount)
        {
            //ROM中のデータの展開・自動逆アセンブルを行う
            //ROM固有展開処理を行う場合はのSampleROMDataSet.ReloadROMImageを変更すること
            ds.ReloadROMImage(ref NodeCount);
        }


        /// <summary>
        /// 画像データのデコードを行う
        /// 画像データの格納方法、展開方法はゲームごとに異なるために独自のデコードプログラムを書く必要がある
        /// </summary>
        /// <returns></returns>
        public Bitmap DecodeBitmap()
        {
            Bitmap DrawnImage = null;
            try
            {
                //現在選択されているツリーノード名を取得する
                string SelectedNodeName = PluginHost.ROMTreeV.SelectedNode.Text;
                //現在表示されているインデックス()を取得する
                int index = UtilClass.ParseInt(PluginHost.PicIndex);

                //現在選択されているコンボボックスのインデックスを取得する
                ComboBox opcb = PluginHost.OptionComboBox;
                int cbindex = 0;
                try
                {
                    cbindex = UtilClass.ParseInt(opcb.SelectedText);
                }
                catch
                {
                }


/*
                //下記はDQ3,6でのコーディングの一例 「キャラクターグラフィック画像」ノードが選択されている場合は
                //キャラクターグラフィック画像のデコードを行う
                if (SelectedNodeName == DQ36TableNames.CharacterGraphicImage)
                {
                    DrawnImage = ds.GetCharacterBitmap(index, out str, out paletteid);
                    opcb.Text = string.Empty;
                    opcb.Enabled = false;
                }
 */
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            return DrawnImage;
        }


        /// <summary>
        /// ツリービューの別ノードに移動する直前に呼ばれる
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public bool MovingToNode(TreeNode node)
        {
            return true;
        }


        /// <summary>
        /// ツリービューのノードを移動した直後に呼ばれる
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public bool MoveToNode(TreeNode node)
        {
            return false;
        }

        /// <summary>
        /// ツリーノードの選択が変わった直後にHost～呼び出される
        /// </summary>
        /// <param name="node"></param>
        /// <returns>true:Plugin側で処理済 false:Host側で要処理</returns>
        public bool MovedToNode(TreeNode node)
        {
            return true;
        }


        /// <summary>
        /// データ編集終了直前に呼ばれる
        /// ユーザーが入力したデータが正しくない場合はe.cancelにtrueをセットする
        /// </summary>
        /// <param name="dt">編集対象のテーブル</param>
        /// <param name="rowindex">編集中のインデックス</param>
        /// <param name="e">ユーザー入力中のデータを保持するクラス e.FormattedValue.ToString()でユーザーの入力値を取得できる</param>
        /// <returns>true:Plugin側でチェック済み false:Host側で要チェック</returns>
        public bool CellValidating_DataV(DataTable dt, int rowindex, ref DataGridViewCellValidatingEventArgs e)
        {
/*
            //コーディング例
            if(CheckFunction(dt, rowindex, e.FormattedValue.ToString())
            {
                MessageBox.Show("入力が正しくありません");
                e.Cancel = true;
            }
            return true;
*/
            return false;
        }

        /// <summary>
        /// データの更新を行う
        /// </summary>
        /// <param name="dr">変更対象のDataRow</param>
        /// <param name="index">変更対象のDataRowのDataTable中のインデックス</param>
        public void UpdateData(DataRow dr, int index)
        {
            //以下の行は削除してはいけない
            //可変長データ(文字列など)を編集可能にしている場合はROMDataSet.UpdateDataをオーバーライドして
            //独自の処理を追加で行う必要がある
            ds.UpdateData(dr, index);
        }

        /// <summary>
        /// UpDateDataが呼ばれた直後にHostからコールされる 不要かも 
        /// </summary>
        /// <param name="TableName"></param>
        public void UpdateInfo(string TableName)
        {
        }

        /// <summary>
        /// データグリッドビューの選択されている行が変わったときに呼び出される
        /// </summary>
        /// <param name="dt">対象のDataTable</param>
        /// <param name="index">新しく選択された行のインデックス</param>
        /// <returns>true:Plugin側で処理済み false:Host側で要処理</returns>
        public bool SelectionChanged_DataV(DataTable dt, int index)
        {
            return false;
        }


        /// <summary>
        /// データの要素数を変更する
        /// </summary>
        /// <param name="dt">対象のDataTable</param>
        /// <param name="count"></param>
        /// <returns>true:Plugin側で処理済 false:Host側で要処理</returns>
        public bool ChangeElementSize(DataTable dt, int count)
        {
            return false;
        }


        /// <summary>
        /// 新レコードを途中に挿入する
        /// </summary>
        /// <param name="dt">対象のDataTable</param>
        /// <param name="count"></param>
        /// <returns>true:Plugin側で処理済 false:Host側で要処理</returns>
        public bool InsertElementSize(DataTable dt, int count)
        {
            return false;
        }


        /// <summary>
        /// 途中のレコードを削除する
        /// </summary>
        /// <param name="dt">対象のDataTable</param>
        /// <param name="count"></param>
        /// <returns>true:Plugin側で処理済 false:Host側で要処理</returns>
        public bool RemoveElementSize(DataTable dt, int count)
        {
            return false;
        }


        /// <summary>
        /// メニュー「編集」をユーザーが選んだときの処理
        /// 必要に応じてメニューのEnable/Disableを行う
        /// </summary>
        /// <param name="EditMenu"></param>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void Edit_DropDownOpening(ToolStripMenuItem EditMenu, object sender, EventArgs e)
        {
/*
            //コーディング例 DQ36の場合「モンスター画像」ノードが選択されている場合に
            //「モンスターグラフィックをエクスポート」メニューを「編集」メニューの最後に追加する
            string SelectedNodeName = PluginHost.ROMTreeV.SelectedNode.Text;
            if (SelectedNodeName == DQ36TableNames.MonsterGraphicImage)
            {
                EditMenu.DropDownItems[EditMenu.DropDownItems.Count - 1].Visible = true;
                EditMenu.DropDownItems.Add(MongraExport);
            }
 */ 
        }


        /// <summary>
        /// 画像表示中のコンボボックスの選択が変わったときにHostからコールされる
        /// </summary>
        /// <param name="CurrentBitmap">現在表示しているビットマップ</param>
        /// <returns>新しく表示するビットマップ</returns>
        public Bitmap SelectionChanged_OptionComboBox(Bitmap CurrentBitmap)
        {
            return CurrentBitmap;
        }

        /// <summary>
        /// セルの値のチェックを独自に行いたい時に実装する
        /// 必要なければ変更しなくて良い
        /// </summary>
        /// <param name="dt"></param>
        /// <param name="rowindex"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        public bool CellValidating_DataV(DataDT dt, int rowindex, ref DataGridViewCellValidatingEventArgs e)
        {
            return false;
        }

        /// <summary>
        /// データの更新時に独自の処理を行いたい場合に実装する(文字列操作など可変長データ部など)
        /// 必要なければ変更しなくて良い
        /// </summary>
        /// <param name="dt"></param>
        /// <param name="index"></param>
        public void UpdateData(DataDT dt, int index)
        {
            ds.UpdateData(dt, index);
        }


        /// <summary>
        /// データグリッドの選択状態が変更された場合に独自の処理を行いたい場合に実装する
        /// 必要なければ変更しなくて良い
        /// </summary>
        /// <param name="dt"></param>
        /// <param name="index"></param>
        /// <returns></returns>
        public bool SelectionChanged_DataV(DataDT dt, int index)
        {
            return false;
        }

        /// <summary>
        /// 固定長データの要素数変更時に独自の処理を行いたい場合に実装する
        /// 必要なければ変更しなくて良い
        /// </summary>
        /// <param name="dt"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public bool ChangeElementSize(DataDT dt, int count)
        {
            return false;
        }

        /// <summary>
        /// 固定長データの要素数変更時に独自の処理を行いたい場合に実装する
        /// 必要なければ変更しなくて良い
        /// </summary>
        /// <param name="dt"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public bool InsertElementSize(DataDT dt, int count)
        {
            return false;
        }

        /// <summary>
        /// 固定長データの要素数変更時に独自の処理を行いたい場合に実装する
        /// 必要なければ変更しなくて良い
        /// </summary>
        /// <param name="dt"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public bool RemoveElementSize(DataDT dt, int count)
        {
            return false;
        }



        public bool ReadRomMapXML(string FilePath, ref ROMTreeDataNode XMLDataNodeTop, ref TreeNode RealProgramNodeTop, ref List<TreeNode> UserDefinedProgramNodeList)
        {
            RomMapXMLReader xr = new RomMapXMLReader();
            xr.ds = this.ds;
            xr.XMLDataNodeTop = XMLDataNodeTop;
            xr.RealProgramNodeTop = RealProgramNodeTop;
            xr.ProgramNodeList = UserDefinedProgramNodeList;

            if (xr.ReadXMLFile(FilePath)) return true;

            return false;
        }


        public void SaveRomMapXML(string FilePath, ROMTreeDataNode XMLDataNodeTop, TreeNode RealProgramNodeTop)
        {
            RomMapXMLWriter rmxml = new RomMapXMLWriter();
            rmxml.ds = this.ds;
            rmxml.XMLDataNodeTop = XMLDataNodeTop;
            rmxml.RealProgramNodeTop = RealProgramNodeTop;

            rmxml.WriteXMLFile(FilePath);
        }

        public void RefreshPDTables(List<PreDefinedDT> pdts)
        {

        }
    }
}
